# Propeller 2 USB Keyboard/Mouse Demo v0.1.4
Demonstration clients for the USB host + Keyboard/Mouse `"Boot Protocol"` driver P2 object.

## Features
- Builds with PNut (v34z, Win, P2 RevB+), Fastspin/Flexgui (v4.3.1, Win, Linux, P2 RevB+).
- Support for the P2 Eval `Control` push-button add-on board.
- P2 frequency and USB pin assignments are defined by the client object.
- The P2 sysclock can be changed "on the fly" and the USB object will adjust to the new frequency. The client must post a command event to the USB to `suspend` prior to the frequency change and an event to `resume` the USB after the new frequency is established. The **setSysClock(freq, mode)** method in the top object demonstrates this sequence.
- USB reset, suspend and resume events may be triggered by the client.
- The clients demonstrate basic keyboard and mouse event handling.

## Build/run
All required source files are in the working directory to facilitate build/run by PNut or FastSpin/Flexgui. The top objects are the *KbMObjDemo.spin2 files. Pin assignments are based on P2 Eval board pin groups. See the `Usage` section for default settings. Flexgui/Fastspin/Loadp2 and PNut testing was done on Windows 10, and Flexgui on Linux Ubuntu 20.4. On both platforms the Flexgui default build/run settings were used and serial baud was set to 2_000_000. Serial I/O is via `"JonnyMac's"` very nice `jm_serial.spin2` and `jm_nstr.spin2` objects.

## Usage
The only source file that may require edits are the *KbMObjDemo.spin2 top objects.
- Default clock frequency: 180_000_000MHz.
- Default serial baud: 2_000_000.
- CON **USB0_BASEPIN**: Set to the first pin number of the `USB Serial Host` add-on board pin group (default is pin **16**). The first four pins map to the upper **A** USB port and the remaining four pins map to the lower **B** port.
- VAR **termType**: Set to one of the constants **ANSI** (default) or **PST**.
- P2 Eval `Control` add-on board: supported by the 2xKbmObjDemo.Spin2 client. Specify the pin group in the top object (default is **ctrl.BASEPIN_24**). If you are not using this add-on board you can search on **Parallax Control** and comment out or remove the relevant code block that follows the comment. This will reduce the binary size by about 900 bytes.

Low-speed USB devices should function with a sysclock of 50Mhz and higher. For full-speed devices, 96MHz is recommended as the minimum. In general, low cost devices tend to be the most tolerant.

If a keyboard is connected, there are a handful of key-triggered commands that I use for testing. This key list is available with the 2xKbMObjDemo.spin2 client. The other two client demos don't have the full list available to reduce code size.
- CTRL + A: If the ANSI/VT* terminal type is set, output sequences are emitted to display some ANSII art (2xKbMObjDemo.Spin2 only). One of the three file encodings should work on most terminals. The original source is available at https://16colo.rs/pack/blocktronics_blockalypse/rad-LOVE.ANS.
- CTRL + E: Clears the terminal window and positions the cursor at the home position.
- CTRL + P: Sets the next available USB port that will receive USB reset and suspend/resume actions. If a Parallax Control add-on board is configured, buttons 0 (reset device) and 1 (suspend/resume) may be used.
- CTRL + ALT + SHIFT + R: Triggers a USB `reset` event.
- CTRL + ALT + SHIFT + S: If the currently selected sysclocks table value differs from the current `CLKFREQ` value, the P2 clock is set to the new value. Connected devices will be put into the suspend state prior to the frequency change and resumed after. To fully sync to the new frequency, connected devices will be reset and enumerated.
- CTRL + LeftArrow: Selects the previous entry of the sysclocks table and displays the frequency and mode values. If it is currently positioned at the first entry, it will wrap to the end of the table.
- CTRL + RightArrow: Selects the next entry of the sysclocks table.

If a mouse is connected, the button and movement data will be output to the terminal.

If the `Control` add-on board is connected (2xKbMObjDemo.Spin2 only), the four buttons may be used as listed below. Note that the `CtrlAddOn.spin2` object maps the button indexing to match the pin group that it is connected to. Button zero is always at the upper-left position and button three is always lower-right. "Up" is the P2 Eval board edge that has the `PARALLAX INC` logo.
- Button 0: If a device is connected to the linked port, a USB bus reset is triggered for that device.
- Button 1: If a device is connected to the linked port, the first press/release will trigger a `suspend` command event to the USB and a second press triggers a `resume` command event.
- Button 2: Each press will display the next frequency/mode entry in the `sysclocks` table, or wrap to the first entry if at the end of the table.
- Button 3: Press to set the P2 clock to the current `sysclocks` table entry.


# NOTES:
The default for this demo is to call the usb.key() method for the key data, which returns data similar to the P1 Keyboard.spin object. If you are currently using code from the Spin2USBKbM-v0_02 version, the usb.rawKey() method will return key data in that format, which may make the transition to this version a little less painful.
